"use strict";

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree.
 */
var axios = require('axios');

var AWSSignature = require('react-native-aws-signature');

var getRequestParams = function getRequestParams(service, region, payload) {
  var keys = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  if (!region) throw new Error('Missing required argument: \'region\' is required.');
  if (!service) throw new Error('Missing required argument: \'method\' is required.');
  var awsSignature = new AWSSignature();
  var date = awsSignature.formatDateTime(new Date(Date.now()).toISOString());
  var body = JSON.stringify(payload, null, 0);
  var host = "logs.".concat(region, ".amazonaws.com");
  var headers = {
    Host: host,
    'X-Amz-Date': date,
    'region': region,
    'Content-Length': body.length,
    Accept: 'application/json',
    'Content-Type': 'application/x-amz-json-1.1',
    'X-Amz-Target': "Logs_20140328.".concat(service)
  };
  var credentials = {
    SecretKey: keys.secretAccessKey,
    AccessKeyId: keys.accessKeyId
  };
  var options = {
    path: '/',
    method: 'post',
    service: 'logs',
    headers: headers,
    region: region,
    body: body,
    credentials: credentials
  };
  awsSignature.setParams(options);
  var authorization = awsSignature.getAuthorizationHeader();
  return {
    uri: "https://".concat(host, "/"),
    headers: _objectSpread({}, headers, authorization)
  };
};

var createLogStream = function createLogStream(logStreamName, _ref) {
  var logGroupName = _ref.logGroupName,
      region = _ref.region,
      accessKeyId = _ref.accessKeyId,
      secretAccessKey = _ref.secretAccessKey,
      local = _ref.local;
  if (local && local != 'false') return Promise.resolve({
    message: 'No need to create stream. Local mode is on.'
  });
  if (!logGroupName) throw new Error('Missing required argument: \'logGroupName\' is required.');
  if (!logStreamName) throw new Error('Missing required argument: \'logStreamName\' is required.');
  if (!secretAccessKey) throw new Error('Missing required argument: \'secretAccessKey\' is required.');
  if (!accessKeyId) throw new Error('Missing required argument: \'accessKeyId\' is required.');
  if (!region) throw new Error('Missing required argument: \'region\' is required.');
  var service = 'CreateLogStream';
  var payload = {
    logGroupName: logGroupName,
    logStreamName: logStreamName
  };

  var _getRequestParams = getRequestParams(service, region, payload, {
    accessKeyId: accessKeyId,
    secretAccessKey: secretAccessKey
  }),
      uri = _getRequestParams.uri,
      headers = _getRequestParams.headers;

  var request = axios.create({
    baseURL: uri,
    headers: headers
  });
  return request.post('', payload).then(function (results) {
    console.log(results.data);
    return results.data;
  }).catch(function (err) {
    throw new Error(err.response.data.message);
  });
};

var _sequenceTokens = new Map();
/**
 * Adds logs to a AWS CloudWatch log stream
 * @param  {String|Object|Array} 	entry         	If type is Object it must be structured as follow: { message: ..., timestamp: ... }
 *                                               	where 'message' must be a string and 'timestamp' must be an UTC date number
 *                                               	(e.g. Date.now())
 *                                               	If type is Array, then each item must either be a string or an object { message: ..., timestamp: ... }
 *                                               	following the same rules as above.
 * @param  {String} 				logGroupName  	[description]
 * @param  {String} 				logStreamName 	[description]
 * @param  {String} 				region        	[description]
 * @param  {Object} 				keys          	{ accessKeyId: ..., secretAccessKey: ... }
 * @param  {String} 				sequenceToken 	[description]
 * @param  {Number} 				retryCount    	[description]
 * @return {Promise}               					[description]
 */


var addLogsToStream = function addLogsToStream(entry, logGroupName, logStreamName, region) {
  var keys = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : {};
  var sequenceToken = arguments.length > 5 && arguments[5] !== undefined ? arguments[5] : '';
  var retryCount = arguments.length > 6 && arguments[6] !== undefined ? arguments[6] : 0;
  if (!logGroupName) throw new Error('Missing required argument: \'logGroupName\' is required.');
  if (!logStreamName) throw new Error('Missing required argument: \'logStreamName\' is required.');
  if (!entry) throw new Error('Missing required argument: \'entry\' is required.');

  var entryType = _typeof(entry);

  var now = Date.now();
  var events = // entry is a message
  entryType == 'string' ? [{
    message: entry,
    timestamp: now
  }] : // entry is a well-formatted log event
  entryType == 'object' && entry.timestamp ? [{
    message: entry.message,
    timestamp: entry.timestamp
  }] : // entry is an array of items
  entry.length > 0 ? entry.map(function (e) {
    return typeof e == 'string' ? {
      message: e,
      timestamp: now
    } : e.timestamp ? {
      message: e.message,
      timestamp: e.timestamp
    } : null;
  }).filter(function (x) {
    return x;
  }) : null;
  var nothingToLog = events == null;
  var service = 'PutLogEvents';
  var payload = {
    logEvents: events,
    logGroupName: logGroupName,
    logStreamName: logStreamName
  };
  var tokenKey = logGroupName + '__' + logStreamName;
  sequenceToken = sequenceToken || _sequenceTokens.get(tokenKey);
  if (sequenceToken) payload.sequenceToken = sequenceToken;

  var _getRequestParams2 = getRequestParams(service, region, payload, keys),
      uri = _getRequestParams2.uri,
      headers = _getRequestParams2.headers;

  var request = axios.create({
    baseURL: uri,
    headers: headers
  });
  return retryCount > 3 || nothingToLog ? Promise.resolve(null) : request.post('', payload).then(function (results) {
    var token = results.data.nextSequenceToken;

    _sequenceTokens.set(tokenKey, token);
  }).catch(function (err) {
    console.log('Oops');
    console.log(err);
    var token = err.response.data.expectedSequenceToken;

    if (token) {
      _sequenceTokens.set(tokenKey, token);

      retryCount += 1;
      return addLogsToStream(entry, logGroupName, logStreamName, region, keys, token, retryCount);
    } else console.error(err.response.data);
  });
};

var delayFn = function delayFn(fn, time) {
  return makeQuerablePromise(new Promise(function (onSuccess) {
    return setTimeout(function () {
      return onSuccess();
    }, time);
  }).then(function () {
    return fn();
  }));
};

var makeQuerablePromise = function makeQuerablePromise(promise) {
  // Don't modify any promise that has been already modified.
  if (promise.isResolved) return promise; // Set initial state

  var isPending = true;
  var isRejected = false;
  var isFulfilled = false; // Observe the promise, saving the fulfillment in a closure scope.

  var result = promise.then(function (v) {
    isFulfilled = true;
    isPending = false;
    return v;
  }, function (e) {
    isRejected = true;
    isPending = false;
    throw e;
  });

  result.isFulfilled = function () {
    return isFulfilled;
  };

  result.isPending = function () {
    return isPending;
  };

  result.isRejected = function () {
    return isRejected;
  };

  return result;
};

var _logbuffer = new WeakMap();

var Logger = function Logger(_ref2) {
  var _this = this;

  var logGroupName = _ref2.logGroupName,
      logStreamName = _ref2.logStreamName,
      region = _ref2.region,
      accessKeyId = _ref2.accessKeyId,
      secretAccessKey = _ref2.secretAccessKey,
      uploadFreq = _ref2.uploadFreq,
      local = _ref2.local;

  _classCallCheck(this, Logger);

  if (!logGroupName) throw new Error('Missing required argument: \'logGroupName\' is required.');
  if (!region) throw new Error('Missing required argument: \'region\' is required.');
  if (!secretAccessKey) throw new Error('Missing required argument: \'secretAccessKey\' is required.');
  if (!accessKeyId) throw new Error('Missing required argument: \'accessKeyId\' is required.');
  var keys = accessKeyId && secretAccessKey ? {
    accessKeyId: accessKeyId,
    secretAccessKey: secretAccessKey
  } : {};
  var log;

  if (!uploadFreq || uploadFreq < 0) {
    log = function log() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      var ls = args[0] && args[0].logStreamName ? args[0].logStreamName : logStreamName;
      if (!ls) throw new Error('Missing required argument: \'logStreamName\' is required.');
      args.shift();
      var logs = (args || []).map(function (x) {
        return JSON.stringify(x);
      }); // console.log('Logging now...')
      // console.log(logs)

      addLogsToStream(logs, logGroupName, ls, region, keys);
    };
  } else {
    log = function log() {
      for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        args[_key2] = arguments[_key2];
      }

      // 1. Accumulate all logs
      var ls = args[0] && args[0].logStreamName ? args[0].logStreamName : logStreamName;
      if (!ls) throw new Error('Missing required argument: \'logStreamName\' is required.');
      args.shift();
      var now = Date.now();
      var logs = (args || []).map(function (x) {
        return {
          message: JSON.stringify(x),
          timestamp: now
        };
      });
      var latestBuffer = _logbuffer.get(_this) || {
        latest: now,
        data: [],
        job: null
      };
      latestBuffer.data = latestBuffer.data.concat(logs); // 2. If no job has ever been started, start it, or if the job is ready to process more

      if (!latestBuffer.job || !latestBuffer.job.isPending()) {
        latestBuffer.job = makeQuerablePromise(delayFn(function () {
          var _ref3 = _logbuffer.get(_this) || {
            latest: now,
            data: [],
            job: null
          },
              latest = _ref3.latest,
              data = _ref3.data,
              job = _ref3.job; // console.log('Finally logging now...')
          // console.log(data)


          _logbuffer.set(_this, {
            latest: latest,
            data: [],
            job: job
          });

          addLogsToStream(data, logGroupName, ls, region, keys);
        }, uploadFreq));
      } //console.log('Buffering logs now...')
      // 3. In any case, memoize


      _logbuffer.set(_this, latestBuffer);
    };
  }

  this.log = local && local != 'false' ? console.log : log;
};

module.exports = {
  createLogStream: createLogStream,
  Logger: Logger
};